@@ -31,14 +31,14 @@ import sys
 def read_plain(_, fpath, fname):
     if fname == "-":
         return sys.stdin.read()
-    with open(fpath) as f:
+    with open(fpath, encoding="utf-8") as f:
         return f.read()
 
 def read_json(_, fpath, fname):
     from json import load
     if fname == "-":
         return load(sys.stdin)
-    with open(fpath) as f:
+    with open(fpath, encoding="utf-8") as f:
         return load(f)
 
 def parse_coq_plain(contents):
@@ -69,7 +69,8 @@ def register_docutils(v, sertop_args):
     setup()
     return v
 
-def _gen_docutils_html(source, fpath, webpage_style, no_header,
+def _gen_docutils_html(source, fpath,
+                       webpage_style, include_banner, include_vernums,
                        html_assets, traceback, Parser, Reader):
     from docutils.core import publish_string
     from .docutils import HtmlTranslator, HtmlWriter
@@ -85,7 +86,8 @@ def _gen_docutils_html(source, fpath, webpage_style, no_header,
         'embed_stylesheet': False,
         'stylesheet_path': None,
         'stylesheet_dirs': [],
-        'no_header': no_header,
+        'alectryon_banner': include_banner,
+        'alectryon_vernums': include_vernums,
         'webpage_style': webpage_style,
         'input_encoding': 'utf-8',
         'output_encoding': 'utf-8'
@@ -102,15 +104,23 @@ def _gen_docutils_html(source, fpath, webpage_style, no_header,
         settings_overrides=settings_overrides, config_section=None,
         enable_exit_status=True).decode("utf-8")
 
-def gen_rstcoq_html(coq, fpath, webpage_style, no_header, html_assets, traceback):
+def gen_rstcoq_html(coq, fpath, webpage_style,
+                    include_banner, include_vernums,
+                    html_assets, traceback):
     from .docutils import RSTCoqParser, RSTCoqStandaloneReader
-    return _gen_docutils_html(coq, fpath, webpage_style, no_header, html_assets, traceback,
+    return _gen_docutils_html(coq, fpath, webpage_style,
+                         include_banner, include_vernums,
+                         html_assets, traceback,
                          RSTCoqParser, RSTCoqStandaloneReader)
 
-def gen_rst_html(rst, fpath, webpage_style, no_header, html_assets, traceback):
+def gen_rst_html(rst, fpath, webpage_style,
+                 include_banner, include_vernums,
+                 html_assets, traceback):
     from docutils.parsers.rst import Parser
     from docutils.readers.standalone import Reader
-    return _gen_docutils_html(rst, fpath, webpage_style, no_header, html_assets, traceback,
+    return _gen_docutils_html(rst, fpath, webpage_style,
+                         include_banner, include_vernums,
+                         html_assets, traceback,
                          Parser, Reader)
 
 def _docutils_cmdline(description, Reader, Parser):
@@ -163,7 +173,7 @@ def _scrub_fname(fname):
     import re
     return re.sub("[^-a-zA-Z0-9]", "-", fname)
 
-def gen_html_snippets(annotated, fname):
+def gen_html_snippets(annotated, include_vernums, fname):
     from .html import HtmlGenerator
     from .pygments import highlight_html
     return HtmlGenerator(highlight_html, _scrub_fname(fname)).gen(annotated)
@@ -178,30 +188,31 @@ COQDOC_OPTIONS = ['--body-only', '--no-glob', '--no-index', '--no-externals',
 
 def _run_coqdoc(coq_snippets, coqdoc_bin=None):
     """Get the output of coqdoc on coq_code."""
-    from tempfile import mkstemp
+    from shutil import rmtree
+    from tempfile import mkstemp, mkdtemp
     from subprocess import check_output
     coqdoc_bin = coqdoc_bin or os.path.join(os.getenv("COQBIN", ""), "coqdoc")
-    fd, filename = mkstemp(prefix="coqdoc_", suffix=".v")
+    dpath = mkdtemp(prefix="alectryon_coqdoc_")
+    fd, filename = mkstemp(prefix="alectryon_coqdoc_", suffix=".v", dir=dpath)
     try:
         for snippet in coq_snippets:
             os.write(fd, snippet.encode("utf-8"))
             os.write(fd, b"\n(* --- *)\n") # Separator to prevent fusing
         os.close(fd)
-        coqdoc = [coqdoc_bin, *COQDOC_OPTIONS, filename]
-        return check_output(coqdoc, timeout=10).decode("utf-8")
+        coqdoc = [coqdoc_bin, *COQDOC_OPTIONS, "-d", dpath, filename]
+        return check_output(coqdoc, cwd=dpath, timeout=10).decode("utf-8")
     finally:
-        os.remove(filename)
+        rmtree(dpath)
 
-def _gen_coqdoc_html(coqdoc_comments):
+def _gen_coqdoc_html(coqdoc_fragments):
     from bs4 import BeautifulSoup
-    coqdoc_output = _run_coqdoc(coqdoc_comments)
+    coqdoc_output = _run_coqdoc(fr.contents for fr in coqdoc_fragments)
     soup = BeautifulSoup(coqdoc_output, "html.parser")
     docs = soup.find_all(class_='doc')
-    if len(docs) != len(coqdoc_comments):
+    if len(docs) != sum(1 for c in coqdoc_fragments if not c.special):
         from pprint import pprint
         print("Coqdoc mismatch:", file=sys.stderr)
-        pprint(docs)
-        pprint(coqdoc_comments)
+        pprint(list(zip(coqdoc_comments, docs)))
         raise AssertionError()
     return docs
 
@@ -213,18 +224,19 @@ def _gen_html_snippets_with_coqdoc(annotated, fname):
 
     writer = HtmlGenerator(highlight_html, _scrub_fname(fname))
 
-    coqdoc = [part.contents for fragments in annotated
-              for part in isolate_coqdoc(fragments)
+    parts = [part for fragments in annotated
+             for part in isolate_coqdoc(fragments)]
+    coqdoc = [part for part in parts
               if isinstance(part, CoqdocFragment)]
     coqdoc_html = iter(_gen_coqdoc_html(coqdoc))
 
-    for fragments in annotated:
-        for part in isolate_coqdoc(fragments):
-            if isinstance(part, CoqdocFragment):
+    for part in parts:
+        if isinstance(part, CoqdocFragment):
+            if not part.special:
                 yield [raw(str(next(coqdoc_html, None)))]
-            else:
-                fragments = default_transform(part.fragments)
-                yield writer.gen_fragments(fragments)
+        else:
+            fragments = default_transform(part.fragments)
+            yield writer.gen_fragments(fragments)
 
 def gen_html_snippets_with_coqdoc(annotated, html_classes, fname):
     html_classes.append("coqdoc")
@@ -237,18 +249,24 @@ def copy_assets(state, html_assets, copy_fn, output_directory):
         cp(output_directory, assets=html_assets, copy_fn=copy_fn)
     return state
 
-def dump_html_standalone(snippets, fname, webpage_style, no_header, html_assets, html_classes):
+def dump_html_standalone(snippets, fname, webpage_style,
+                         include_banner, include_vernums,
+                         html_assets, html_classes):
     from dominate import tags, document
     from dominate.util import raw
+    from . import GENERATOR
     from .core import SerAPI
-    from .html import gen_header, wrap_classes, GENERATOR, ASSETS
     from .pygments import HTML_FORMATTER
+    from .html import ASSETS, ADDITIONAL_HEADS, gen_banner, wrap_classes
 
     doc = document(title=fname)
-    doc.head.add(tags.meta(charset="utf-8"))
-    doc.head.add(tags.meta(name="generator", content=GENERATOR))
     doc.set_attribute("class", "alectryon-standalone")
 
+    doc.head.add(tags.meta(charset="utf-8"))
+    doc.head.add(tags.meta(name="generator", content=GENERATOR))
+
+    for hd in ADDITIONAL_HEADS:
+        doc.head.add(raw(hd))
     for css in ASSETS.ALECTRYON_CSS:
         doc.head.add(tags.link(rel="stylesheet", href=css))
     for link in (ASSETS.IBM_PLEX_CDN, ASSETS.FIRA_CODE_CDN):
@@ -264,8 +282,8 @@ def dump_html_standalone(snippets, fname, webpage_style, no_header, html_assets,
 
     cls = wrap_classes(webpage_style, *html_classes)
     root = doc.body.add(tags.article(cls=cls))
-    if not no_header:
-        root.add(raw(gen_header(SerAPI.version_info())))
+    if include_banner:
+        root.add(raw(gen_banner(SerAPI.version_info(), include_vernums)))
     for snippet in snippets:
         root.add(snippet)
 
@@ -305,7 +323,7 @@ def write_output(ext, contents, fname, output, output_directory):
     else:
         if not output:
             output = os.path.join(output_directory, strip_extension(fname) + ext)
-        with open(output, mode="w") as f:
+        with open(output, mode="w", encoding="utf-8") as f:
             f.write(contents)
 
 def write_file(ext):
@@ -507,9 +525,15 @@ and produce reStructuredText, HTML, or JSON output.""")
                         help=CACHE_DIRECTORY_HELP)
 
     NO_HEADER_HELP = "Do not insert a header with usage instructions in webpages."
-    parser.add_argument("--no-header", action='store_true',
+    parser.add_argument("--no-header", action='store_false',
+                        dest="include_banner", default="True",
                         help=NO_HEADER_HELP)
 
+    NO_VERSION_NUMBERS = "Omit version numbers in meta tags and headers."
+    parser.add_argument("--no-version-numbers", action='store_false',
+                        dest="include_vernums", default=True,
+                        help=NO_VERSION_NUMBERS)
+
     WEBPAGE_STYLE_HELP = "Choose a style for standalone webpages."
     WEBPAGE_STYLE_CHOICES = ("centered", "floating", "windowed")
     parser.add_argument("--webpage-style", default="centered",
@@ -546,6 +570,9 @@ and produce reStructuredText, HTML, or JSON output.""")
                       metavar=("DIR", "COQDIR"), nargs=2, action="append",
                       default=[], help=R_HELP)
 
+    EXPECT_UNEXPECTED_HELP = "Ignore unexpected output from SerAPI"
+    parser.add_argument("--expect-unexpected", action="store_true",
+                        default=False, help=EXPECT_UNEXPECTED_HELP)
 
     DEBUG_HELP = "Print communications with SerAPI."
     parser.add_argument("--debug", action="store_true",
@@ -594,6 +621,10 @@ def process_pipelines(args):
         from . import docutils
         docutils.CACHE_DIRECTORY = args.cache_directory
 
+    if args.expect_unexpected:
+        from . import core
+        core.SerAPI.EXPECT_UNEXPECTED = True
+
     try:
         for fpath, pipeline in args.pipelines:
             state, ctx = None, build_context(fpath, args)
